/* 
calc_soilconc.c
calculation of soil concentration data (for all layer, if layerFlag=-1 and for given layer, if 0<=layer<10)
calculation of groundwater and flooding concentration data, if data is != -1
content2pool_flag = 0: NH4, NO3, DOC, DON - > content_array
content2pool_flag = 1: content_array -> NH4, NO3, DOC, DON

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v7.0.
Copyright 2021, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_func.h"
#include "bgc_constants.h"
int calc_soilconc(int layerFlag, int content2pool_flag, const soilprop_struct* sprop, const wstate_struct* ws, cstate_struct* cs, nstate_struct* ns, soilInfo_struct* soilInfo)
{
	int errorCode, layer, layerS, layerE, dm;
	errorCode = 0;

	/* calculation for all layers (layerFlag == -1) or only the given layer (layerFlag > -1) */
	if (layerFlag == -1)
	{
		layerS = 0;
		layerE = N_SOILLAYERS;
	}
	else
	{
		layerS = layerFlag;
		layerE = layerFlag + 1;
	}

	for (layer = layerS; layer < layerE; layer++)
	{
		if (content2pool_flag == 0)
		{
			soilInfo->content_soil[0][layer] = ns->NH4[layer];
			soilInfo->content_soil[1][layer] = ns->NO3[layer];
			soilInfo->content_soil[2][layer] = ns->soil1n[layer];
			soilInfo->content_soil[3][layer] = ns->soil2n[layer];
			soilInfo->content_soil[4][layer] = ns->soil3n[layer];
			soilInfo->content_soil[5][layer] = ns->soil4n[layer];
			soilInfo->content_soil[6][layer] = cs->soil1c[layer];
			soilInfo->content_soil[7][layer] = cs->soil2c[layer];
			soilInfo->content_soil[8][layer] = cs->soil3c[layer];
			soilInfo->content_soil[9][layer] = cs->soil4c[layer];
		}
		else
		{
			ns->NH4[layer] = soilInfo->content_soil[0][layer];
			ns->NO3[layer] = soilInfo->content_soil[1][layer];
			ns->soil1n[layer] = soilInfo->content_soil[2][layer];
			ns->soil2n[layer] = soilInfo->content_soil[3][layer];
			ns->soil3n[layer] = soilInfo->content_soil[4][layer];
			ns->soil4n[layer] = soilInfo->content_soil[5][layer];
			cs->soil1c[layer] = soilInfo->content_soil[6][layer];
			cs->soil2c[layer] = soilInfo->content_soil[7][layer];
			cs->soil3c[layer] = soilInfo->content_soil[8][layer];
			cs->soil4c[layer] = soilInfo->content_soil[9][layer];
		}

		
		/* control for negative pool */
		for (dm = 0; dm < N_DISSOLVMATER; dm++)
		{
			if (!errorCode && soilInfo->content_soil[dm][layer] < 0.0)
			{
				if (fabs(soilInfo->content_soil[dm][layer]) > CRIT_PREC && !errorCode)
				{
					printf("\n");
					printf("ERROR: negative nitrogen pool (calc_soilconc.c)\n");
					errorCode = 1;
				}
			}

			if (ws->soilw[layer])
				soilInfo->conc_soil[dm][layer] = soilInfo->content_soil[dm][layer] / ws->soilw[layer] * soilInfo->dissolv_prop[layer];
			else
				soilInfo->conc_soil[dm][layer] = 0;
		}


		/* control for unsat CF-layer is it is avaialbe  */
		if (sprop->GWlayer != DATA_GAP && layer == (int)sprop->CFlayer && sprop->dz_zoneNORM_unsat && content2pool_flag == 1)
		{
			for (dm = 0; dm < N_DISSOLVMATER; dm++)
			{
				if (soilInfo->content_zoneNORM_unsat[dm] < 0)
				{
					if (fabs(soilInfo->content_zoneNORM_unsat[dm]) < CRIT_PREC)
					{
						soilInfo->content_soil[dm][layer] -= soilInfo->content_zoneNORM_unsat[dm];
						soilInfo->content_zoneNORM_unsat[dm] = 0;
					}
					else
					{
						printf("\n");
						printf("ERROR: negative pool in zone_NORM (calc_soilconc.c)\n");
						errorCode = 1;
					}
				}

				if (soilInfo->content_zoneCAPIL_unsat[dm] < 0)
				{
					if (fabs(soilInfo->content_zoneCAPIL_unsat[dm]) < CRIT_PREC)
					{
						soilInfo->content_soil[dm][layer] -= soilInfo->content_zoneCAPIL_unsat[dm];
						soilInfo->content_zoneCAPIL_unsat[dm] = 0;
					}
					else
					{
						printf("\n");
						printf("ERROR: negative pool in zone_CAPIL (calc_soilconc.c)\n");
						errorCode = 1;
					}
				}

		
				if (!errorCode && soilInfo->content_soil[dm][layer] - (soilInfo->content_zoneNORM_unsat[dm] + soilInfo->content_zoneCAPIL_unsat[dm]) > CRIT_PREC * 10)
				{
					printf("\n");
					printf("ERROR: sum of zone_NORM, zone_CAPIL  is not equal to content of unsat CFlayer (calc_soilconc.c)\n");
					errorCode = 1;
				}




			}
		}

		/* control for GWlayer */
		if (sprop->GWlayer != DATA_GAP  && layer == (int)sprop->GWlayer && content2pool_flag == 1)
		{
			for (dm = 0; dm < N_DISSOLVMATER; dm++)
			{
				if (soilInfo->content_zoneNORM[dm] < 0)
				{ 
					if (fabs(soilInfo->content_zoneNORM[dm]) < CRIT_PREC)
					{
						soilInfo->content_soil[dm][layer] -= soilInfo->content_zoneNORM[dm];
						soilInfo->content_zoneNORM[dm] = 0;
					}
					else
					{
						printf("\n");
						printf("ERROR: negative pool in zone_NORM (calc_soilconc.c)\n");
						errorCode = 1;
					}
				}

				if (soilInfo->content_zoneCAPIL[dm] < 0)
				{
					if (fabs(soilInfo->content_zoneCAPIL[dm]) < CRIT_PREC)
					{
						soilInfo->content_soil[dm][layer] -= soilInfo->content_zoneCAPIL[dm];
						soilInfo->content_zoneCAPIL[dm] = 0;
					}
					else
					{
						printf("\n");
						printf("ERROR: negative pool in zone_CAPIL (calc_soilconc.c)\n");
						errorCode = 1;
					}
				}

				if (soilInfo->content_zoneSAT[dm] < 0)
				{
					if (fabs(soilInfo->content_zoneSAT[dm]) < CRIT_PREC)
					{
						soilInfo->content_soil[dm][layer] -= soilInfo->content_zoneSAT[dm];
						soilInfo->content_zoneSAT[dm] = 0;
					}
					else
					{
						printf("\n");
						printf("ERROR: negative pool in zone_SAT (calc_soilconc.c)\n");
						errorCode = 1;
					}
				}
				if (!errorCode && soilInfo->content_soil[dm][layer] - (soilInfo->content_zoneNORM[dm] + soilInfo->content_zoneCAPIL[dm] + soilInfo->content_zoneSAT[dm]) > CRIT_PREC*10)
				{
					printf("\n");
					printf("ERROR: sum of zone_NORM, zone_CAPIL and zone_SAT is not equal to content of GWlayer (calc_soilconc.c)\n");
					errorCode = 1;
				}

		
			

			}
		}

		

	
	}



	return(errorCode);
}

