/* 
groundwater_leaching.c
Calculating soil mineral nitrogen and DOC-DON leaching in groundwater layers 
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v7.0.
Copyright 2022, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_func.h"
#include "bgc_constants.h"

int capillary_leaching(control_struct* ctrl, soilprop_struct* sprop, soilInfo_struct* soilInfo, cstate_struct* cs, nstate_struct* ns, wstate_struct* ws, wflux_struct* wf)
{
	int errorCode, GWlayer, CFlayer, dm, layer;

	double wflux, wstate0, wstate1;
	double state0[N_DISSOLVMATER], state1[N_DISSOLVMATER], conc0[N_DISSOLVMATER], conc1[N_DISSOLVMATER], leachFlux[N_DISSOLVMATER];

	GWlayer = (int)sprop->GWlayer;
	CFlayer = (int)sprop->CFlayer;
	errorCode = 0;




	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* I. between zoneSAT vs. zoneCAP -  */

	wflux = wf->GWrecharge - wf->GWdischarge[GWlayer];

	if (wflux != 0)
	{
		/* if there is capillary zone in GW-layer*/
		if (sprop->dz_zoneCAPIL)
		{ 
			wstate0 = sprop->soilw_zoneCAPIL;
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state0[dm] = soilInfo->content_zoneCAPIL[dm];

			wstate1 = sprop->soilw_zoneSAT;
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state1[dm] = soilInfo->content_zoneSAT[dm];


			if (!errorCode && leachCalc(ctrl, soilInfo, wflux, wstate0, wstate1, state0, state1, conc0, conc1, leachFlux))
			{
				printf("\n");
				printf("ERROR in leachCalc.c for groundwater_leaching.c\n");
				errorCode = 1;
			}

			for (dm = 0; dm < N_DISSOLVMATER; dm++)
			{
				soilInfo->content_zoneCAPIL[dm] = state0[dm];

				if (leachFlux[dm] > 0)
					soilInfo->dismatGWrecharge[dm] = leachFlux[dm];
				else
					soilInfo->dismatGWdischarge[dm][GWlayer] = -1 * leachFlux[dm];

				soilInfo->content_soil[dm][GWlayer] -= leachFlux[dm];
			}
		}
		else
		{
			wstate0 = ws->soilw[GWlayer-1];
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state0[dm] = soilInfo->content_soil[dm][GWlayer - 1];

			wstate1 = sprop->soilw_zoneSAT;
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state1[dm] = soilInfo->content_zoneSAT[dm];


			if (!errorCode && leachCalc(ctrl, soilInfo, wflux, wstate0, wstate1, state0, state1, conc0, conc1, leachFlux))
			{
				printf("\n");
				printf("ERROR in leachCalc.c for groundwater_leaching.c\n");
				errorCode = 1;
			}

			for (dm = 0; dm < N_DISSOLVMATER; dm++)
			{
				soilInfo->content_soil[dm][GWlayer - 1] = state0[dm];
				soilInfo->conc_soil[dm][GWlayer - 1] = conc0[dm];

				if (leachFlux[dm] > 0)
					soilInfo->dismatGWrecharge[dm] = leachFlux[dm];
				else
					soilInfo->dismatGWdischarge[dm][GWlayer] = -1 * leachFlux[dm];

			//	soilInfo->content_soil[dm][GWlayer] -= leachFlux[dm];
			}
		}
	}

	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* II. If GWlayer has three sub-layers: SAT, CAPIL and NORM */


	if (sprop->dz_zoneNORM)
	{	

		/* II./.1 zone_NORM and zone_CAPIL */
		wflux = wf->soilwFlux_NORMvsCAPIL;

		if (wflux != 0)
		{
			wstate0 = sprop->soilw_zoneNORM;
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state0[dm] = soilInfo->content_zoneNORM[dm];

			wstate1 = sprop->soilw_zoneCAPIL;
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state1[dm] = soilInfo->content_zoneCAPIL[dm];


			if (!errorCode && leachCalc(ctrl, soilInfo, wflux, wstate0, wstate1, state0, state1, conc0, conc1, leachFlux))
			{
				printf("\n");
				printf("ERROR in leachCalc.c for groundwater_leaching.c\n");
				errorCode = 1;
			}

			for (dm = 0; dm < N_DISSOLVMATER; dm++)
			{
				soilInfo->content_zoneNORM[dm] = state0[dm];
				soilInfo->content_zoneCAPIL[dm] = state1[dm];
				soilInfo->dismatLeach_NORM[dm] = leachFlux[dm];
			}
		}
		/* II/.2 last noGW-layer and zone_NORM*/
		wflux = wf->soilwFlux[GWlayer - 1];

		if (wflux != 0)
		{
			wstate0 = ws->soilw[GWlayer - 1];
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state0[dm] = soilInfo->content_soil[dm][GWlayer - 1];

			wstate1 = sprop->soilw_zoneNORM;
			for (dm = 0; dm < N_DISSOLVMATER; dm++) state1[dm] = soilInfo->content_zoneNORM[dm];


			if (!errorCode && leachCalc(ctrl, soilInfo, wflux, wstate0, wstate1, state0, state1, conc0, conc1, leachFlux))
			{
				printf("\n");
				printf("ERROR in leachCalc.c for groundwater_leaching.c\n");
				errorCode = 1;
			}

			for (dm = 0; dm < N_DISSOLVMATER; dm++)
			{
				soilInfo->content_soil[dm][GWlayer - 1] = state0[dm];
				soilInfo->conc_soil[dm][GWlayer - 1] = conc0[dm];
				soilInfo->content_zoneNORM[dm] = state1[dm];

				soilInfo->dismatLeach[dm][GWlayer - 1] = leachFlux[dm];

				soilInfo->content_soil[dm][GWlayer] += leachFlux[dm];
			}
		}

	}
	else
	{

		/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		/* III. If the capillary zone spans several layers: direct filling from GW layer only when capillary layers below are satured */


		for (layer = GWlayer - 1; layer > CFlayer; layer--)
		{
			wflux = -wf->GWdischarge[layer];

			if (wflux != 0)
			{
				wstate0 = ws->soilw[layer];
				for (dm = 0; dm < N_DISSOLVMATER; dm++) state0[dm] = soilInfo->content_soil[dm][layer];

				wstate1 = sprop->soilw_zoneSAT;
				for (dm = 0; dm < N_DISSOLVMATER; dm++) state1[dm] = soilInfo->content_zoneSAT[dm];


				if (!errorCode && leachCalc(ctrl, soilInfo, wflux, wstate0, wstate1, state0, state1, conc0, conc1, leachFlux))
				{
					printf("\n");
					printf("ERROR in leachsCalc.c for groundwater_leaching.c\n");
					errorCode = 1;
				}

				for (dm = 0; dm < N_DISSOLVMATER; dm++)
				{


					if (leachFlux[dm] > 0)
					{
						printf("\n");
						printf("ERROR in leachsCalc.c for groundwater_leaching.c\n");
						errorCode = 1;
					}
					else
						soilInfo->dismatGWdischarge[dm][layer] = -1 * leachFlux[dm];

					soilInfo->content_soil[dm][layer] -= leachFlux[dm];
				}
			}
		}

	}


	/* calculation of src/snk variables   */

	for (dm = 0; dm < N_DISSOLVN; dm++)
	{
		ns->GWsnk_N += soilInfo->dismatGWrecharge[dm];
		for (layer = 0; layer < N_SOILLAYERS; layer++) ns->GWsrc_N += soilInfo->dismatGWdischarge[dm][layer];
	}
	for (dm = N_DISSOLVN; dm < N_DISSOLVMATER; dm++)
	{
		cs->GWsnk_C += soilInfo->dismatGWrecharge[dm];
		for (layer = 0; layer < N_SOILLAYERS; layer++) cs->GWsrc_C += soilInfo->dismatGWdischarge[dm][layer];
	}

	/* transfer value: content_array ->NH4, NO3, DOC, DON  etc.*/
	if (!errorCode && calc_soilconc(-1, 1, sprop, ws, cs, ns, soilInfo))
	{
		printf("ERROR in calc_soilconc.c for groundwater_leaching.c\n");
		errorCode = 1;
	}

	return (errorCode);
}

