/*
capillary_tipping.c
Calculation of percolation and diffusion fluxes 

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v7.0.
Copyright 2022, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_func.h"
#include "bgc_constants.h"
#define MIN(X, Y) (((X) < (Y)) ? (X) : (Y))
#define MAX(X, Y) (((X) > (Y)) ? (X) : (Y))

int capillary_tipping(siteconst_struct* sitec, soilprop_struct* sprop, epvar_struct* epv, wstate_struct* ws, wflux_struct* wf)
{

	int errorCode = 0;
	int  ll, layer;

	double VWC, soilw_sat1, soilw1;
	double INFILT, conduct_cmday, conductSAT_cmday;
	double VWCsat, VWCfc, dz0, dz0_cm, dz1, HOLD, soilw0;

	double DC, DRN, EXCESS, VWCnew;

	int CFlayer = (int)sprop->CFlayer;
	int GWlayer = (int)sprop->GWlayer;


	/* infiltration: percolation from CFlayer above */
	if (CFlayer == 0)
		INFILT = wf->infiltPOT * mm_to_cm;
	else
		INFILT = wf->soilwPercol[CFlayer - 1] * mm_to_cm;

	/* saturated hydraulic conductivity in actual CFlayer (cm/day = m/s * 100 * sec/day) */
	conduct_cmday = sprop->hydrCONDUCTsat[CFlayer] * pow(epv->VWC[CFlayer] / sprop->VWCsat[CFlayer], 2 * sprop->soilB[CFlayer] + 3) * m_to_cm * nSEC_IN_DAY;
	conduct_cmday = sprop->hydrCONDUCTsat[CFlayer] * m_to_cm * nSEC_IN_DAY;

	/* -----------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* 1. normZone in CFlayer without GW */

	if (sprop->dz_zoneNORM_unsat > 0)
	{

		VWC = sprop->VWC_zoneNORM_unsat;
		VWCsat = sprop->VWCsat[CFlayer];
		VWCfc = sprop->VWCfc_base[CFlayer];
		soilw0 = sprop->soilw_zoneNORM_unsat;

		/* unit change from m to cm */
		dz0 = sprop->dz_zoneNORM_unsat;
		dz0_cm = dz0 * m_to_cm;

		DC = sprop->drainCoeff[CFlayer];

		/* hydraulic conductivity in actual CFlayer (cm/day = m/s * 100 * sec/day) */
		conductSAT_cmday = sprop->hydrCONDUCTsat[CFlayer] * m_to_cm * nSEC_IN_DAY;
		conduct_cmday = sprop->hydrCONDUCTsat[CFlayer] * pow(epv->VWC[CFlayer] / sprop->VWCsat[CFlayer], 2 * sprop->soilB[CFlayer] + 3) * m_to_cm * nSEC_IN_DAY;
		if (conduct_cmday < conductSAT_cmday / 10.) conduct_cmday = conductSAT_cmday / 10.;


		if (!errorCode && calc_drainage(wf->flagRAIN, INFILT, VWC, VWCsat, VWCfc, dz0_cm, DC, conduct_cmday, &DRN, &EXCESS, &VWCnew))
		{
			printf("\n");
			printf("ERROR calc_drainage.c for capillary_tipping.c\n");
			errorCode = 1;
		}

		/* water flux: cm/day to kg/(m2*day) */
		wf->soilwPercol_NORMvsCAPIL_unsat = DRN / mm_to_cm;

		/* state update: with new VWC calcualte soilw */
		sprop->VWC_zoneNORM_unsat = VWCnew;
		sprop->soilw_zoneNORM_unsat = sprop->VWC_zoneNORM_unsat * dz0 * water_density;

		/* control */
		if (fabs(VWCnew * dz0 * water_density - soilw0 - ((INFILT - DRN - EXCESS) / mm_to_cm)) > CRIT_PRECwater)
		{
			printf("\n");
			printf("ERROR in capillary_tipping.c: soilw udpate\n");
			errorCode = 1;
		}

		INFILT = DRN;




		/* if there is excess water, redistribute it in layers above */
		if (EXCESS > 0)
		{
			for (ll = CFlayer - 1; ll >= 0; ll--)
			{
				dz1 = sitec->soillayer_thickness[ll];
				soilw_sat1 = sprop->VWCsat[ll] * dz1 * water_density;
				soilw1 = epv->VWC[ll] * dz1 * water_density;
				HOLD = MIN((soilw_sat1 - soilw1) * mm_to_cm, EXCESS);
				soilw1 += HOLD / mm_to_cm;
				if (soilw1 > soilw_sat1)
				{
					printf("\n");
					printf("ERROR in capillary_tipping.c: EXCESS calculation\n");
					errorCode = 1;
				}
				ws->soilw[ll] = soilw1;
				epv->VWC[ll] = ws->soilw[ll] / dz1 / water_density;

				wf->soilwPercol[ll] -= EXCESS / mm_to_cm;

				EXCESS = EXCESS - HOLD;

			}
			/* if too much pondwater -> runoff */
			wf->soilw_to_pondw += EXCESS / mm_to_cm;;
		}
	}

	/* -----------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* 2. capillZone in CFlayer without GW  */

	if (sprop->dz_zoneCAPIL_unsat)
	{
		VWC = sprop->VWC_zoneCAPIL_unsat;
		VWCsat = sprop->VWCsat[CFlayer];
		VWCfc = sprop->VWCfc[CFlayer];
		soilw0 = sprop->soilw_zoneCAPIL_unsat;

		/* unit change from m to cm */
		dz0 = sprop->dz_zoneCAPIL_unsat;
		dz0_cm = dz0 * m_to_cm;

		DC = sprop->drainCoeff[CFlayer];

		/* hydraulic conductivity in actual CFlayer (cm/day = m/s * 100 * sec/day) */
		conductSAT_cmday = sprop->hydrCONDUCTsat[CFlayer] * m_to_cm * nSEC_IN_DAY;
		conduct_cmday = sprop->hydrCONDUCTsat[CFlayer] * pow(epv->VWC[CFlayer] / sprop->VWCsat[CFlayer], 2 * sprop->soilB[CFlayer] + 3) * m_to_cm * nSEC_IN_DAY;
		if (conduct_cmday < conductSAT_cmday / 10.) conduct_cmday = conductSAT_cmday / 10.;


		if (!errorCode && calc_drainage(wf->flagRAIN, INFILT, VWC, VWCsat, VWCfc, dz0_cm, DC, conduct_cmday, &DRN, &EXCESS, &VWCnew))
		{
			printf("\n");
			printf("ERROR calc_drainage.c for capillary_tipping.c\n");
			errorCode = 1;
		}

		/* water flux: cm/day to kg/(m2*day) */
		wf->soilwPercol[CFlayer] = DRN / mm_to_cm;
		

		/* state update: with new VWC calcualte soilw */
		sprop->VWC_zoneCAPIL_unsat = VWCnew;
		sprop->soilw_zoneCAPIL_unsat = sprop->VWC_zoneCAPIL_unsat * dz0 * water_density;

		/* control */
		if (fabs(VWCnew * dz0 * water_density - soilw0 - ((INFILT - DRN - EXCESS) / mm_to_cm)) > CRIT_PRECwater)
		{
			printf("\n");
			printf("ERROR in capillary_tipping.c: soilw update\n");
			errorCode = 1;
		}


		INFILT = DRN;

		/* if there is excess water, redistribute it in layers above */
		if (EXCESS > 0)
		{
			/* first of all: normal layer is saturated */
			dz1 = sprop->soilw_zoneNORM_unsat;
			soilw_sat1 = sprop->VWCsat[CFlayer] * dz1 * water_density;
			soilw1 = sprop->VWC_zoneNORM_unsat * dz1 * water_density;
			HOLD = MIN((soilw_sat1 - soilw1) * mm_to_cm, EXCESS);
			soilw1 += HOLD / mm_to_cm;
			if (soilw1 > soilw_sat1)
			{
				printf("\n");
				printf("ERROR in tipping.c: EXCESS calculation\n");
				errorCode = 1;
			}
			sprop->soilw_zoneNORM_unsat = soilw1;
			sprop->VWC_zoneNORM_unsat = sprop->soilw_zoneNORM_unsat / dz1 / water_density;

			wf->soilwPercol_NORMvsCAPIL_unsat -= EXCESS / mm_to_cm;

			for (ll = CFlayer - 1; ll >= 0; ll--)
			{
				dz1 = sitec->soillayer_thickness[ll];
				soilw_sat1 = sprop->VWCsat[ll] * dz1 * water_density;
				soilw1 = epv->VWC[ll] * dz1 * water_density;
				HOLD = MIN((soilw_sat1 - soilw1) * mm_to_cm, EXCESS);
				soilw1 += HOLD / mm_to_cm;
				if (soilw1 > soilw_sat1)
				{
					printf("\n");
					printf("ERROR in tipping.c: EXCESS calculation\n");
					errorCode = 1;
				}
				ws->soilw[ll] = soilw1;
				epv->VWC[ll] = ws->soilw[ll] / sitec->soillayer_thickness[ll] / water_density;

				wf->soilwPercol[ll] -= EXCESS / mm_to_cm;

				EXCESS = EXCESS - HOLD;

			}
			/* if too much pondwater -> runoff */
			wf->soilw_to_pondw += EXCESS / mm_to_cm;;
		}




		/* state update: with new VWC calcualte soilw */
		ws->soilw[CFlayer] = sprop->soilw_zoneNORM_unsat + sprop->soilw_zoneCAPIL_unsat;
		epv->VWC[CFlayer] = ws->soilw[CFlayer] / (sitec->soillayer_thickness[CFlayer] * water_density);
	}



	/* -----------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* 3. Capillary layers betwen CFlayer and GWlayer */

	INFILT = wf->soilwPercol[CFlayer];

	for (layer = CFlayer+1; layer < GWlayer; layer++)
	{

		VWC = epv->VWC[layer];
		VWCsat = sprop->VWCsat[layer];
		VWCfc = sprop->VWCfc[layer];
		soilw0 = ws->soilw[layer];

		/* unit change from m to cm */
		dz0 = sitec->soillayer_thickness[layer];
		dz0_cm = dz0 * m_to_cm;

		DC = sprop->drainCoeff[layer];

		/* hydraulic conductivity in actual layer (cm/day = m/s * 100 * sec/day) */
		conductSAT_cmday = sprop->hydrCONDUCTsat[layer] * m_to_cm * nSEC_IN_DAY;
		conduct_cmday = sprop->hydrCONDUCTsat[layer] * pow(epv->VWC[layer] / sprop->VWCsat[layer], 2 * sprop->soilB[layer] + 3) * m_to_cm * nSEC_IN_DAY;
		if (conduct_cmday < conductSAT_cmday / 10.) conduct_cmday = conductSAT_cmday / 10.;


		if (!errorCode && calc_drainage(wf->flagRAIN, INFILT, VWC, VWCsat, VWCfc, dz0_cm, DC, conduct_cmday, &DRN, &EXCESS, &VWCnew))
		{
			printf("\n");
			printf("ERROR calc_drainage.c for tipping.c\n");
			errorCode = 1;
		}

		/* water flux: cm/day to kg/(m2*day) */
		wf->soilwPercol[layer] = DRN / mm_to_cm;


		/* state update: with new VWC calcualte soilw */
		epv->VWC[layer] = VWCnew;
		ws->soilw[layer] = epv->VWC[layer] * sitec->soillayer_thickness[layer] * water_density;
		
		/* control */
		if (fabs(VWCnew * dz0 * water_density - soilw0 - ((INFILT - DRN - EXCESS) / mm_to_cm)) > CRIT_PRECwater)
		{
			printf("\n");
			printf("ERROR in capillary_tipping.c: soilw update\n");
			errorCode = 1;
		}

		INFILT = DRN;

		/* if there is excess water, redistribute it in layers above */
		if (EXCESS > 0)
		{
			for (ll = layer - 1; ll >= 0; ll--)
			{
				//balus: megoldani a NORM-CAPIL feltltdst
				errorCode = 1;
				dz1 = sitec->soillayer_thickness[ll];
				soilw_sat1 = sprop->VWCsat[ll] * dz1 * water_density;
				soilw1 = epv->VWC[ll] * dz1 * water_density;
				HOLD = MIN((soilw_sat1 - soilw1) * mm_to_cm, EXCESS);
				soilw1 += HOLD / mm_to_cm;
				if (soilw1 > soilw_sat1)
				{
					printf("\n");
					printf("ERROR in capillary_tipping.c: EXCESS calculation\n");
					errorCode = 1;
				}
				ws->soilw[ll] = soilw1;
				epv->VWC[ll] = ws->soilw[ll] / dz1 / water_density;

				wf->soilwPercol[ll] -= EXCESS / mm_to_cm;

				EXCESS = EXCESS - HOLD;

			}
			/* if too much pondwater -> runoff */
			wf->soilw_to_pondw += EXCESS / mm_to_cm;;
		}



	} /* END FOR (layer) */



	return (errorCode);

}
