/*
groundwater_diffusion.c
UPWARD WATER MOVEMENT in groundwater layers

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v7.0.
Copyright 2024, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_func.h"
#include "bgc_constants.h"
#define MIN(X, Y) (((X) < (Y)) ? (X) : (Y))
#define MAX(X, Y) (((X) > (Y)) ? (X) : (Y))

int groundwater_diffusion(siteconst_struct* sitec, soilprop_struct* sprop, epvar_struct* epv, wstate_struct* ws, wflux_struct* wf)
{

	int errorCode = 0;
	int GWlayer, CFlayer, layer, flagSAT, layerSAT;

	double soilwDiffus_act;
	double VWC0, VWC0_sat, VWC0_fc, VWC0_wp, VWC1, VWC1_sat, VWC1_fc, VWC1_wp;
	double rVWC0, rVWC1, rVWC_limit, VWC0_limit, VWC1_limit, soillayer_midpoint;

	double dz0, dz1, dLk;

	double ratioNORM, ratioCAPIL;
	double K_sum, K_mean, K_eff,  diffusGW_sum;
	double diffusGW[N_SOILLAYERS];

	for (layer = 0; layer < N_SOILLAYERS; layer++) diffusGW[layer] = 0;

	soilwDiffus_act = K_sum = K_mean = diffusGW_sum = ratioNORM = ratioCAPIL = 0;
	layerSAT = 0;
	GWlayer = (int)sprop->GWlayer;
	CFlayer = (int)sprop->CFlayer;
	flagSAT = 1; 


	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* I. between zoneSAT vs. zoneCAP -  */

	if (sprop->dz_zoneCAPIL)
	{
		dz0 = sprop->dz_zoneCAPIL;
		VWC0 = sprop->VWC_zoneCAPIL;
		VWC0_sat = sprop->VWCsat[GWlayer];
		VWC0_fc = sprop->VWCsat[GWlayer];
		VWC0_wp = sprop->VWCwp[GWlayer];


		dz1 = sprop->dz_zoneSAT;
		VWC1 = sprop->VWCsat[GWlayer];
		VWC1_sat = sprop->VWCsat[GWlayer];
		VWC1_fc = sprop->VWCsat[GWlayer];
		VWC1_wp = sprop->VWCwp[GWlayer];


		/* no limitation of relative VWC */
		VWC0_limit = sprop->VWCsat[GWlayer];
		VWC1_limit = 0;


		dLk = DATA_GAP;


		if (!errorCode && calc_diffus(GWlayer, sprop, dz0, VWC0, VWC0_sat, VWC0_fc, VWC0_wp, VWC0_limit, dz1, VWC1, VWC1_sat, VWC1_fc, VWC1_wp, VWC1_limit, dLk, &soilwDiffus_act))
		{
			printf("\n");
			printf("ERROR in calc_diffus.c for groundwater_diffusion.c\n");
			errorCode = 1;
		}

		if (fabs(soilwDiffus_act) < CRIT_PRECwater) soilwDiffus_act = 0;


		if (soilwDiffus_act > 0)
		{
			printf("\n");
			printf("ERROR in groundwater_diffusion.c: no diffusion from zoneCAPIL to zoneSAT.c\n");
			errorCode = 1;
		}

		/* calculation of limitation from Ksat to direct filling */

		K_eff = MAX(0, sprop->hydrCONDUCTsat[layer] * 86400 * 1000);

		/* limitation  */
		if (fabs(soilwDiffus_act) > K_eff)
			wf->GWdischarge[GWlayer] += K_eff;
		else
			wf->GWdischarge[GWlayer] += -1 * soilwDiffus_act;


		sprop->soilw_zoneCAPIL -= soilwDiffus_act;
		sprop->VWC_zoneCAPIL = sprop->soilw_zoneCAPIL / dz0 / water_density;

		ws->soilw[GWlayer] -= soilwDiffus_act;
		epv->VWC[GWlayer] = ws->soilw[GWlayer] / sitec->soillayer_thickness[GWlayer] / water_density;

		/* flagSAT: saturated or not? */
		if (sprop->VWCsat[layer] - sprop->VWC_zoneCAPIL > CRIT_PRECwater)
			flagSAT = 0;
		else
			flagSAT = 1;
	}



	layerSAT = GWlayer - 1;

	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
	/* II. If GWlayer has three sub-layers: SAT, CAPIL and NORM */

	if (sprop->dz_zoneNORM)
	{
		/* control */
		if (GWlayer != CFlayer)
		{
			printf("\n");
			printf("ERROR in sublayer differenciation in groundwater_diffusion.c\n");
			errorCode = 1;
		}

		/* --------------------------------------------------------------------------------------------------------*/
		/* II./2. between zoneCAP vs. zoneNORM  */
	
		dz0 = sprop->dz_zoneNORM;
		VWC0 = sprop->VWC_zoneNORM;
		VWC0_sat = sprop->VWCsat[GWlayer];
		VWC0_fc = sprop->VWCfc_base[GWlayer];
		VWC0_wp = sprop->VWCwp[GWlayer];

		dz1 = sprop->dz_zoneCAPIL;
		VWC1 = sprop->VWC_zoneCAPIL;
		VWC1_sat = sprop->VWCsat[GWlayer];
		VWC1_fc = sprop->VWCfc_base[GWlayer];
		VWC1_wp = sprop->VWCwp[GWlayer];

		rVWC0 = (VWC0 - VWC0_wp) / (VWC0_fc - VWC0_wp);
		rVWC1 = (VWC1 - VWC1_wp) / (VWC1_fc - VWC1_wp);
		rVWC_limit = (rVWC0 + rVWC1) / 2;
		VWC0_limit = rVWC_limit * (VWC0_fc - VWC0_wp) + VWC0_wp;
		VWC1_limit = rVWC_limit * (VWC1_fc - VWC1_wp) + VWC1_wp;


		if (!errorCode && calc_diffus(GWlayer, sprop, dz0, VWC0, VWC0_sat, VWC0_fc, VWC0_wp, VWC0_limit, dz1, VWC1, VWC1_sat, VWC1_fc, VWC1_wp, VWC1_limit, dLk, &soilwDiffus_act))
		{
			printf("\n");
			printf("ERROR in calc_diffus.c for tipping.c\n");
			errorCode = 1;
		}

		if (fabs(soilwDiffus_act) < CRIT_PRECwater) soilwDiffus_act = 0;


		wf->soilwDiffus_NORMvsCAPIL += soilwDiffus_act;


		sprop->soilw_zoneNORM -= soilwDiffus_act;
		sprop->soilw_zoneCAPIL += soilwDiffus_act;

		sprop->VWC_zoneNORM = sprop->soilw_zoneNORM / dz0 / water_density;
		sprop->VWC_zoneCAPIL = sprop->soilw_zoneCAPIL / dz1 / water_density;

		/* --------------------------------------------------------------------------------------------------------*/
		/* I./2. between zoneCAP+zoneNORM  vs noGWGWlayer*/

		if (GWlayer != 0)
		{
			dz0 = sitec->soillayer_thickness[GWlayer - 1];
			VWC0 = epv->VWC[GWlayer - 1];
			VWC0_sat = sprop->VWCsat[GWlayer - 1];
			VWC0_fc = sprop->VWCfc_base[GWlayer - 1];
			VWC0_wp = sprop->VWCwp[GWlayer - 1];

			/* if there is capillary zone in GWlayer: diffusion between CAPIL+NORM*/
			if (sprop->dz_zoneCAPIL)
			{ 
				dz1 = sprop->dz_zoneNORM + sprop->dz_zoneCAPIL;
				ratioNORM = sprop->dz_zoneNORM / (sprop->dz_zoneNORM + sprop->dz_zoneCAPIL);
				ratioCAPIL = sprop->dz_zoneCAPIL / (sprop->dz_zoneNORM + sprop->dz_zoneCAPIL);
				VWC1 = sprop->VWC_zoneNORM * ratioNORM +sprop->VWC_zoneCAPIL * ratioCAPIL;
				VWC1_sat = sprop->VWCsat[GWlayer];
				VWC1_fc = sprop->VWCfc_base[GWlayer];
				VWC1_wp = sprop->VWCwp[GWlayer];
			}
			/* if there is not capillary zone in GWlayer: diffusion between SAT and unsat layer above */
			else
			{
				dz1 = sitec->soillayer_thickness[GWlayer];
				VWC1 = sprop->VWCsat[GWlayer];
				VWC1_sat = sprop->VWCsat[GWlayer];
				VWC1_fc = sprop->VWCfc_base[GWlayer];
				VWC1_wp = sprop->VWCwp[GWlayer];
				ratioNORM = 0;
				ratioCAPIL = 0;
			}

			rVWC0 = (VWC0 - VWC0_wp) / (VWC0_fc - VWC0_wp);
			rVWC1 = (VWC1 - VWC1_wp) / (VWC1_fc - VWC1_wp);
			rVWC_limit = (rVWC0 + rVWC1) / 2;
			VWC0_limit = rVWC_limit * (VWC0_fc - VWC0_wp) + VWC0_wp;
			VWC1_limit = rVWC_limit * (VWC1_fc - VWC1_wp) + VWC1_wp;


			if (!errorCode && calc_diffus(GWlayer, sprop, dz0, VWC0, VWC0_sat, VWC0_fc, VWC0_wp, VWC0_limit, dz1, VWC1, VWC1_sat, VWC1_fc, VWC1_wp, VWC1_limit, dLk, &soilwDiffus_act))
			{
				printf("\n");
				printf("ERROR in calc_diffus.c for tipping.c\n");
				errorCode = 1;
			}

			if (fabs(soilwDiffus_act) < CRIT_PRECwater) soilwDiffus_act = 0;


			
			ws->soilw[GWlayer-1]  -= soilwDiffus_act;
			epv->VWC[GWlayer - 1] = ws->soilw[GWlayer - 1] / dz0 / water_density;

			sprop->soilw_zoneNORM += soilwDiffus_act * ratioNORM;
			sprop->soilw_zoneCAPIL += soilwDiffus_act * ratioCAPIL;
			if (sprop->dz_zoneNORM)  sprop->VWC_zoneNORM = sprop->soilw_zoneNORM / sprop->dz_zoneNORM / water_density;
			if (sprop->dz_zoneCAPIL) sprop->VWC_zoneCAPIL = sprop->soilw_zoneCAPIL / sprop->dz_zoneCAPIL / water_density;

			wf->soilwDiffus[GWlayer-1] = soilwDiffus_act;

	
		}
	
		ws->soilw[GWlayer] = sprop->soilw_zoneNORM + sprop->soilw_zoneCAPIL + sprop->soilw_zoneSAT;
		epv->VWC[GWlayer] = ws->soilw[GWlayer] / sitec->soillayer_thickness[GWlayer] / water_density;
		sprop->NORMlayer = GWlayer - 1;
	}
	else
	{

		/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		/* III. If the capillary zone spans several layers: direct filling from GW layer only when capillary layers below are satured */

		while (flagSAT == 1 && layerSAT > CFlayer)
		{

			dz0 = sitec->soillayer_thickness[layerSAT];
			VWC0 = epv->VWC[layerSAT];
			VWC0_fc = sprop->VWCfc[layerSAT]; 

			VWC0_sat = sprop->VWCsat[layerSAT];
			VWC0_wp = sprop->VWCwp[layerSAT];


			dz1 = sprop->dz_zoneSAT;
			VWC1 = sprop->VWCsat[GWlayer];
			VWC1_sat = sprop->VWCsat[GWlayer];
			VWC1_fc = sprop->VWCsat[GWlayer];
			VWC1_wp = sprop->VWCwp[GWlayer];


			/* no limitation of relative VWC */
			VWC0_limit = sprop->VWCsat[layerSAT];
			VWC1_limit = 0;


			dLk = (sprop->GWD - sitec->soillayer_midpoint[layerSAT]) * m_to_cm;


			if (!errorCode && calc_diffus(layerSAT, sprop, dz0, VWC0, VWC0_sat, VWC0_fc, VWC0_wp, VWC0_limit, dz1, VWC1, VWC1_sat, VWC1_fc, VWC1_wp, VWC1_limit, dLk, &soilwDiffus_act))
			{
				printf("\n");
				printf("ERROR in calc_diffus.c for tipping.c\n");
				errorCode = 1;
			}

			if (fabs(soilwDiffus_act) < CRIT_PRECwater) soilwDiffus_act = 0;


			if (soilwDiffus_act > 0)
			{
				printf("\n");
				printf("ERROR in groundwater_diffusion.c: no diffusion from zoneCAPIL to zoneSAT.c\n");
				errorCode = 1;
			}

			/* calculation of limitation from Ksat to direct filling */
			for (layer = GWlayer; layer > layerSAT; layer--)
			{
				K_sum += sprop->hydrCONDUCTsat[layer];
				diffusGW_sum += diffusGW[layer];
			}
			K_mean = K_sum/ (double)(GWlayer - layerSAT);

			K_eff = MAX(0, K_mean * 86400 * 1000 - fabs(diffusGW_sum));

			/* limitation only if there is intermediate capillary layer : if (fabs(soilwDiffus_act) > K_eff && sprop->CFeff[layerSAT + 1] != 0) */
			if (fabs(soilwDiffus_act) > K_eff)
				diffusGW[layerSAT] = -1 * K_eff;
			else
				diffusGW[layerSAT] = soilwDiffus_act;
			

			/* update of pools */
			ws->soilw[layerSAT] -= diffusGW[layerSAT];
			epv->VWC[layerSAT] = ws->soilw[layerSAT] / sitec->soillayer_thickness[layerSAT] / water_density;

			wf->GWdischarge[layerSAT] = -1 * diffusGW[layerSAT];

			/* flagSAT: saturated or not? */
			if (sprop->VWCsat[layerSAT] - epv->VWC[layerSAT] > CRIT_PRECwater)
			{
				flagSAT = 0;
				sprop->NORMlayer = layerSAT - 1;
			}
			else
				layerSAT -= 1;

		}
		
		/* ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
		/* if below uppermost capillary layer all capillary layers is saturated: diffusion between NORM and CAPIL in unsat capillary zone is calculated here */
		if (flagSAT == 1)
		{
			/* ------------------*/
			/* CAPIl vs SAT */
			dz0 = sprop->dz_zoneCAPIL_unsat;
			VWC0 = sprop->VWC_zoneCAPIL_unsat;
			VWC0_fc = sprop->VWCsat[layerSAT];

			soillayer_midpoint = sitec->soillayer_depth[layerSAT] + dz0 / 2;

			VWC0_sat = sprop->VWCsat[layerSAT];
			VWC0_wp = sprop->VWCwp[layerSAT];


			dz1 = sprop->dz_zoneSAT;
			VWC1 = sprop->VWCsat[GWlayer];
			VWC1_sat = sprop->VWCsat[GWlayer];
			VWC1_fc = sprop->VWCsat[GWlayer];
			VWC1_wp = sprop->VWCwp[GWlayer];


			/* no limitation of relative VWC */
			VWC0_limit = sprop->VWCsat[layerSAT];
			VWC1_limit = 0;


			dLk = (sprop->GWD - soillayer_midpoint) * m_to_cm;


			if (!errorCode && calc_diffus(layerSAT, sprop, dz0, VWC0, VWC0_sat, VWC0_fc, VWC0_wp, VWC0_limit, dz1, VWC1, VWC1_sat, VWC1_fc, VWC1_wp, VWC1_limit, dLk, &soilwDiffus_act))
			{
				printf("\n");
				printf("ERROR in calc_diffus.c for tipping.c\n");
				errorCode = 1;
			}

			if (fabs(soilwDiffus_act) < CRIT_PRECwater) soilwDiffus_act = 0;


			if (soilwDiffus_act > 0)
			{
				printf("\n");
				printf("ERROR in groundwater_diffusion.c: no diffusion from zoneCAPIL to zoneSAT.c\n");
				errorCode = 1;
			}

			/* calculation of limitation from Ksat to direct filling */
			for (layer = GWlayer; layer > layerSAT; layer--)
			{
				K_sum += sprop->hydrCONDUCTsat[layer];
				diffusGW_sum += diffusGW[layer];
			}
		
			K_mean = K_sum / (double)(GWlayer - layerSAT);
			K_eff = MAX(0, K_mean * 86400 * 1000 - fabs(diffusGW_sum));

			/* limitation only if there is intermediate capillary layer : if (fabs(soilwDiffus_act) > K_eff && sprop->CFeff[layerSAT + 1] != 0) */
			if (fabs(soilwDiffus_act) > K_eff)
				diffusGW[layerSAT] = -1 * K_eff;
			else
				diffusGW[layerSAT] = soilwDiffus_act;


			/* update of pools */
			ws->soilw[layerSAT] -= diffusGW[layerSAT];
			epv->VWC[layerSAT] = ws->soilw[layerSAT] / sitec->soillayer_thickness[layerSAT] / water_density;

			wf->GWdischarge[layerSAT] = -1 * diffusGW[layerSAT];

			sprop->soilw_zoneCAPIL_unsat -= diffusGW[layerSAT];
			sprop->VWC_zoneCAPIL_unsat = sprop->soilw_zoneCAPIL_unsat / dz0 / water_density;


			/* ------------------*/
			/* CAPIL vs NORM */

			dz0 = sprop->dz_zoneNORM_unsat;
			VWC0 = sprop->VWC_zoneNORM_unsat;
			VWC0_sat = sprop->VWCsat[layerSAT];
			VWC0_fc = sprop->VWCfc_base[layerSAT];
			VWC0_wp = sprop->VWCwp[layerSAT];

			dz1 = sprop->dz_zoneCAPIL_unsat;
			VWC1 = sprop->VWC_zoneCAPIL_unsat;
			VWC1_sat = sprop->VWCsat[layerSAT];
			VWC1_fc = sprop->VWCfc_base[layerSAT];
			VWC1_wp = sprop->VWCwp[layerSAT];

			rVWC0 = (VWC0 - VWC0_wp) / (VWC0_fc - VWC0_wp);
			rVWC1 = (VWC1 - VWC1_wp) / (VWC1_fc - VWC1_wp);
			rVWC_limit = (rVWC0 + rVWC1) / 2;
			VWC0_limit = rVWC_limit * (VWC0_fc - VWC0_wp) + VWC0_wp;
			VWC1_limit = rVWC_limit * (VWC1_fc - VWC1_wp) + VWC1_wp;


			if (!errorCode && calc_diffus(layerSAT, sprop, dz0, VWC0, VWC0_sat, VWC0_fc, VWC0_wp, VWC0_limit, dz1, VWC1, VWC1_sat, VWC1_fc, VWC1_wp, VWC1_limit, dLk, &soilwDiffus_act))
			{
				printf("\n");
				printf("ERROR in calc_diffus.c for tipping.c\n");
				errorCode = 1;
			}

			if (fabs(soilwDiffus_act) < CRIT_PRECwater) soilwDiffus_act = 0;


			wf->soilwDiffus_NORMvsCAPIL_unsat += soilwDiffus_act;


			sprop->soilw_zoneNORM_unsat -= soilwDiffus_act;
			sprop->soilw_zoneCAPIL_unsat += soilwDiffus_act;

			sprop->VWC_zoneNORM_unsat = sprop->soilw_zoneNORM_unsat / dz0 / water_density;
			sprop->VWC_zoneCAPIL_unsat = sprop->soilw_zoneCAPIL_unsat / dz1 / water_density;


			sprop->NORMlayer = layerSAT;


		}


	}

	



	return (errorCode);

}
