/*
groundwater_tipping.c
Calculation of percolation and diffusion fluxes 

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v7.0.
Copyright 2022, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_func.h"
#include "bgc_constants.h"
#define MIN(X, Y) (((X) < (Y)) ? (X) : (Y))
#define MAX(X, Y) (((X) > (Y)) ? (X) : (Y))

int groundwater_tipping(siteconst_struct* sitec, soilprop_struct* sprop, epvar_struct* epv, wstate_struct* ws, wflux_struct* wf)
{

	int errorCode = 0;
	int  ll;

	double VWC, soilw_sat1, soilw1;
	double INFILT, conduct_cmday, conductSAT_cmday;
	double VWCsat, VWCfc, dz0, dz1, dz0_cm, HOLD, soilw0;

	double DC, DRN, EXCESS, VWCnew;

	int GWlayer = (int)sprop->GWlayer;



	/* infiltration: percolation from layer above */
	if (GWlayer == 0)
		INFILT = wf->infiltPOT * mm_to_cm;
	else
		INFILT = wf->soilwPercol[GWlayer - 1] * mm_to_cm;

	/* hydraulic conductivity in actual GWlayer (cm/day = m/s * 100 * sec/day) */
	conductSAT_cmday = sprop->hydrCONDUCTsat[GWlayer] * m_to_cm * nSEC_IN_DAY;
	conduct_cmday = sprop->hydrCONDUCTsat[GWlayer] * pow(epv->VWC[GWlayer] / sprop->VWCsat[GWlayer], 2 * sprop->soilB[GWlayer] + 3) * m_to_cm * nSEC_IN_DAY;
	if (conduct_cmday < conductSAT_cmday / 10.) conduct_cmday = conductSAT_cmday / 10.;

	/* -----------------------------*/
	/* 1. normZone */

	if (sprop->dz_zoneNORM > 0)
	{

		VWC = sprop->VWC_zoneNORM;
		VWCsat = sprop->VWCsat[GWlayer];
		VWCfc = sprop->VWCfc_base[GWlayer];
		soilw0 = sprop->soilw_zoneNORM;

		/* unit change from m to cm */
		dz0 = sprop->dz_zoneNORM;
		dz0_cm = dz0 * m_to_cm;

		DC = sprop->drainCoeff[GWlayer];


		if (!errorCode && calc_drainage(wf->flagRAIN, INFILT, VWC, VWCsat, VWCfc, dz0_cm, DC, conduct_cmday, &DRN, &EXCESS, &VWCnew))
		{
			printf("\n");
			printf("ERROR calc_drainage.c for tipping.c\n");
			errorCode = 1;
		}

		/* water flux: cm/day to kg/(m2*day) */
		wf->soilwPercol_NORMvsCAPIL = DRN / mm_to_cm;

		/* state update: with new VWC calcualte soilw */
		sprop->VWC_zoneNORM = VWCnew;
		sprop->soilw_zoneNORM = sprop->VWC_zoneNORM * dz0 * water_density;

		/* control */
		if (fabs(VWCnew * dz0 * water_density - soilw0 - ((INFILT - DRN - EXCESS) / mm_to_cm)) > CRIT_PRECwater)
		{
			printf("\n");
			printf("ERROR in tipping.c: soilw udpate\n");
			errorCode = 1;
		}

		INFILT = DRN;




		/* if there is excess water, redistribute it in layers above */
		if (EXCESS > 0)
		{
			for (ll = GWlayer - 1; ll >= 0; ll--)
			{
				dz1 = sitec->soillayer_thickness[ll];
				soilw_sat1 = sprop->VWCsat[ll] * dz1 * water_density;
				soilw1 = epv->VWC[ll] * dz1 * water_density;
				HOLD = MIN((soilw_sat1 - soilw1) * mm_to_cm, EXCESS);
				soilw1 += HOLD / mm_to_cm;
				if (soilw1 > soilw_sat1)
				{
					printf("\n");
					printf("ERROR in tipping.c: EXCESS calculation\n");
					errorCode = 1;
				}
				ws->soilw[ll] = soilw1;
				epv->VWC[ll] = ws->soilw[ll] / dz1 / water_density;

				wf->soilwPercol[ll] -= EXCESS / mm_to_cm;

				EXCESS = EXCESS - HOLD;

			}
			/* if too much pondwater -> runoff */
			wf->soilw_to_pondw += EXCESS / mm_to_cm;;
		}
	}

	/* -----------------------------*/
	/* 2. capillZone */

	if (sprop->dz_zoneCAPIL)
	{
		VWC = sprop->VWC_zoneCAPIL;
		VWCsat = sprop->VWCsat[GWlayer];
		VWCfc = sprop->VWCsat[GWlayer];
		soilw0 = sprop->soilw_zoneCAPIL;

		/* unit change from m to cm */
		dz0 = sprop->dz_zoneCAPIL;
		dz0_cm = dz0 * m_to_cm;

		DC = sprop->drainCoeff[GWlayer];

		/* hydraulic conductivity in actual GWlayer (cm/day = m/s * 100 * sec/day) */
		conductSAT_cmday = sprop->hydrCONDUCTsat[GWlayer] * m_to_cm * nSEC_IN_DAY;
		conduct_cmday = sprop->hydrCONDUCTsat[GWlayer] * pow(epv->VWC[GWlayer] / sprop->VWCsat[GWlayer], 2 * sprop->soilB[GWlayer] + 3) * m_to_cm * nSEC_IN_DAY;
		if (conduct_cmday < conductSAT_cmday / 10.) conduct_cmday = conductSAT_cmday / 10.;


		if (!errorCode && calc_drainage(wf->flagRAIN, INFILT, VWC, VWCsat, VWCfc, dz0_cm, DC, conduct_cmday, &DRN, &EXCESS, &VWCnew))
		{
			printf("\n");
			printf("ERROR calc_drainage.c for tipping.c\n");
			errorCode = 1;
		}

		/* water flux: cm/day to kg/(m2*day) */
		wf->GWrecharge = DRN / mm_to_cm;


		/* state update: with new VWC calcualte soilw */
		sprop->VWC_zoneCAPIL = VWCnew;
		sprop->soilw_zoneCAPIL = sprop->VWC_zoneCAPIL * dz0 * water_density;
		/* control */
		if (fabs(VWCnew * dz0 * water_density - soilw0 - ((INFILT - DRN - EXCESS) / mm_to_cm)) > CRIT_PRECwater)
		{
			printf("\n");
			printf("ERROR in capillary_tipping.c: soilw update\n");
			errorCode = 1;
		}


		INFILT = DRN;

		/* if there is excess water, redistribute it in layers above */
		if (EXCESS > 0)
		{
			/* first of all: normal layer is saturated */
			dz1 = sprop->soilw_zoneNORM;
			soilw_sat1 = sprop->VWCsat[GWlayer] * dz1 * water_density;
			soilw1 = sprop->VWC_zoneNORM * dz1 * water_density;
			HOLD = MIN((soilw_sat1 - soilw1) * mm_to_cm, EXCESS);
			soilw1 += HOLD / mm_to_cm;
			if (soilw1 > soilw_sat1)
			{
				printf("\n");
				printf("ERROR in tipping.c: EXCESS calculation\n");
				errorCode = 1;
			}
			sprop->soilw_zoneNORM = soilw1;
			sprop->VWC_zoneNORM = sprop->soilw_zoneNORM / dz1 / water_density;

			wf->soilwPercol_NORMvsCAPIL -= EXCESS / mm_to_cm;

			for (ll = GWlayer - 1; ll >= 0; ll--)
			{
				dz1 = sitec->soillayer_thickness[ll];
				soilw_sat1 = sprop->VWCsat[ll] * dz1 * water_density;
				soilw1 = epv->VWC[ll] * dz1 * water_density;
				HOLD = MIN((soilw_sat1 - soilw1) * mm_to_cm, EXCESS);
				soilw1 += HOLD / mm_to_cm;
				if (soilw1 > soilw_sat1)
				{
					printf("\n");
					printf("ERROR in tipping.c: EXCESS calculation\n");
					errorCode = 1;
				}
				ws->soilw[ll] = soilw1;
				epv->VWC[ll] = ws->soilw[ll] / sitec->soillayer_thickness[ll] / water_density;

				wf->soilwPercol[ll] -= EXCESS / mm_to_cm;

				EXCESS = EXCESS - HOLD;

			}
			/* if too much pondwater -> runoff */
			wf->soilw_to_pondw += EXCESS / mm_to_cm;;
		}


		/* state update: with new VWC calcualte soilw */
		ws->soilw[GWlayer] = sprop->soilw_zoneNORM + sprop->soilw_zoneCAPIL + sprop->soilw_zoneSAT;
		epv->VWC[GWlayer] = ws->soilw[GWlayer] / (sitec->soillayer_thickness[GWlayer] * water_density);

	}
	else
	{
		/* no capillary zone in GW-layer */
		wf->GWrecharge = INFILT / mm_to_cm;
	}


	return (errorCode);

}
