/* 
pondANDrunoffD.c
state update of pond water, calculation of Dunnian runoff, pond water limitation

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v7.0.
Copyright 2022, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_constants.h"
#include "bgc_func.h"


int pondANDrunoffD(control_struct* ctrl, siteconst_struct* sitec, soilprop_struct* sprop, epvar_struct* epv, wstate_struct* ws, wflux_struct* wf)
{

	/* internal variables */
	int errorCode, layer, flagEXTRA;
	double pondmax, soilwEXTRA, ratio;
	double soilw_zoneNORM_sat, soilw_zoneCAPIL_sat, ratioNORM, ratioCAPIL;

	errorCode=layer=flagEXTRA=0;


	/*--------------------------------------*/
	/* Water flux from soil to pond */
	

	ws->pondw += wf->soilw_to_pondw + wf->prcp_to_pondw;

	/*--------------------------------------*/
	/* Dunnian runoff */
	if (ws->GW_waterlogging > sprop->pondmax)
		pondmax = ws->GW_waterlogging;
	else
		pondmax = sprop->pondmax;

	if (ws->pondw > pondmax)
	{
		wf->pondw_to_runoff = ws->pondw - pondmax;
		ws->pondw = pondmax;
	}



	
	/*--------------------------------------*/
	/* saturation of top soil layers: correction of water flux from pond to soil */

	if (ws->pondw > 0)
	{
		layer = 0;
		flagEXTRA = 0;
		while (flagEXTRA == 0 && layer < N_SOILLAYERS - 1)
		{
			if ((layer == 0 || sprop->infiltDepth_max > sitec->soillayer_depth[layer - 1]))
			{
				if (sprop->infiltDepth_max > sitec->soillayer_depth[layer] || layer == 0)
					soilwEXTRA = (sprop->VWCsat[layer] - epv->VWC[layer]) * sitec->soillayer_thickness[layer] * water_density;
				else
				{
					ratio = ((sprop->infiltDepth_max - sitec->soillayer_depth[layer - 1]) / sitec->soillayer_thickness[layer]);
					soilwEXTRA = ((sprop->VWCsat[layer] - epv->VWC[layer]) * sitec->soillayer_thickness[layer] * water_density) * ratio;
				}
				if (fabs(soilwEXTRA) < CRIT_PRECwater) soilwEXTRA = 0;
				if (soilwEXTRA)
				{
					if (soilwEXTRA > ws->pondw)
						soilwEXTRA = ws->pondw;

					ws->pondw -= soilwEXTRA;
					wf->pondw_to_soilw += soilwEXTRA;

					ws->soilw[layer] += soilwEXTRA;
					epv->VWC[layer] = ws->soilw[layer] / (sitec->soillayer_thickness[layer] * water_density);

					if (layer == (int)sprop->GWlayer && soilwEXTRA)
					{
						soilw_zoneNORM_sat = sprop->VWCsat[layer] * sprop->dz_zoneNORM * water_density;
						soilw_zoneCAPIL_sat = sprop->VWCsat[layer] * sprop->dz_zoneCAPIL * water_density;
						ratioNORM = (soilw_zoneNORM_sat - sprop->soilw_zoneNORM) / soilwEXTRA;
						ratioCAPIL = (soilw_zoneCAPIL_sat - sprop->soilw_zoneCAPIL) / soilwEXTRA;
						sprop->soilw_zoneNORM += soilwEXTRA * ratioNORM;
						sprop->soilw_zoneCAPIL += soilwEXTRA * ratioCAPIL;
						if (sprop->dz_zoneNORM) sprop->VWC_zoneNORM = sprop->soilw_zoneNORM / sprop->dz_zoneNORM / water_density;
						if (sprop->dz_zoneCAPIL) sprop->VWC_zoneCAPIL = sprop->soilw_zoneCAPIL / sprop->dz_zoneCAPIL / water_density;
					}
					if (epv->VWC[layer] - sprop->VWCsat[layer] > CRIT_PRECwater || sprop->VWC_zoneCAPIL - sprop->VWCsat[layer] > CRIT_PRECwater || sprop->VWC_zoneNORM - sprop->VWCsat[layer] > CRIT_PRECwater)
					{
						printf("\n");
						printf("ERROR in pondwANDrunoffD.c for tipping.c\n");
						errorCode = 1;
					}
				}
			}

			else
				flagEXTRA = 1;

			layer += 1;
		}
	}
	

	/*--------------------------------------*/
	/* Pond water evaporation: water stored on surface  */

	if (ws->pondw)
	{
		if (wf->potEVPsurface - wf->EVPsoilw < ws->pondw)
			wf->EVPpondw = wf->potEVPsurface - wf->EVPsoilw;
		else
			wf->EVPpondw = ws->pondw;
	}




	/* Water sink: pond water evaporation */
	if (ws->GW_waterlogging > 0)
		ws->GWsrc_W += wf->EVPpondw;
	else
		ws->pondw -= wf->EVPpondw;



	/* pond_flag: flag of WARNING writing (only at first time) */
	if (!ctrl->pond_flag && ws->pondw) ctrl->pond_flag = 1;


	return (errorCode);
}
