/* 
Elimit_and_PET.c
Calculate the limitation of soil evaporation (actual EVP), GWevap and update top soil water content

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v7.0.
Copyright 2022, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_func.h"
#include "bgc_constants.h"

int soilEVP_calc(control_struct* ctrl, const siteconst_struct* sitec, soilprop_struct* sprop, epvar_struct* epv, wstate_struct* ws, wflux_struct* wf)
{
	int errorCode=0;
	double EVP_lack, soilw_hw0, soilw_diff;
	double ratioNORM, diffNORM, diffCAPIL;
	
	/* SOILEVAP UPDATE: if GW-table is in the top soil layer, the source of evaporation is the GW-table */
	

	if (ws->pondw == 0)
	{
		if ((int)sprop->GWlayer == 0)
		{
			soilw_diff = wf->EVPsoilw - wf->potEVPsurface;
			if (soilw_diff) wf->EVPsoilw -= soilw_diff;
			wf->GWevap = wf->EVPsoilw;
		}
		else
		{
			/* soilw evaporation - limitation: hygroscopic water */
			soilw_hw0 = sprop->VWChw[0] * sitec->soillayer_thickness[0] * water_density;
			EVP_lack = wf->EVPsoilw - (ws->soilw[0] - soilw_hw0);

			/* theoretical lower limit of water content: hygroscopic water content. */
			if (EVP_lack > 0)
			{
				if (ws->EVPsurface2cum >= sprop->soilEVPcrit)
				{
					if (ws->EVPsurface2cum > wf->EVPsoilw)
					{
						ws->EVPsurface2cum = ws->EVPsurface2cum - EVP_lack;
						epv->DSR = pow((ws->EVPsurface2cum / sprop->coeff_EVPcum), 2);
					}
					else
					{
						ws->EVPsurface1cum = ws->EVPsurface1cum - (wf->EVPsoilw - ws->EVPsurface2cum);

						ws->EVPsurface2cum = (ws->soilw[0] - soilw_hw0) - sprop->soilEVPcrit;
						if (ws->EVPsurface2cum < 0) ws->EVPsurface2cum = 0;

						ws->EVPsurface1cum = ws->EVPsurface1cum + (ws->soilw[0] - soilw_hw0);
						if (ws->EVPsurface1cum < sprop->soilEVPcrit) ws->EVPsurface2cum = sprop->soilEVPcrit;

						epv->DSR = pow((ws->EVPsurface2cum / sprop->coeff_EVPcum), 2);
					}
				}
				else
				{
					ws->EVPsurface1cum = ws->EVPsurface1cum - wf->EVPsoilw + (ws->soilw[0] - soilw_hw0);
				}

				wf->EVPsoilw = (ws->soilw[0] - soilw_hw0);
				/* limitEVP_flag: flag of WARNING writing in log file (only at first time) */
				if (fabs(EVP_lack) > CRIT_PREC && !ctrl->limitEVP_flag) ctrl->limitEVP_flag = 1;
			}

			soilw_diff = wf->EVPsoilw - wf->potEVPsurface;

			if (soilw_diff > 0) wf->EVPsoilw -= soilw_diff;

			ws->soilw[0] -= (wf->EVPsoilw);	
			epv->VWC[0] = ws->soilw[0] / water_density / sitec->soillayer_thickness[0];

			/* if capillary zone exists in unsaturated zone (not in GWlayer) and capillary zone is in the top soil layer */
			if (sprop->dz_zoneCAPIL_unsat && sprop->CFlayer == 0)
			{
				ratioNORM = sprop->dz_zoneNORM_unsat / (sprop->dz_zoneCAPIL_unsat + sprop->dz_zoneNORM_unsat);

				diffNORM = wf->EVPsoilw * ratioNORM;

				/* NORM zone: minimum value: hygroscopic water */
				soilw_hw0 = sprop->VWChw[0] * sprop->dz_zoneNORM_unsat * water_density;
				if (sprop->soilw_zoneNORM_unsat + diffNORM < soilw_hw0) diffNORM = sprop->soilw_zoneNORM_unsat - soilw_hw0;

				sprop->soilw_zoneNORM_unsat -= diffNORM;

				diffCAPIL = wf->EVPsoilw - diffNORM;

				/* CAPIL zone: minimum value: hygroscopic water */
				soilw_hw0 = sprop->VWChw[0] * sprop->dz_zoneNORM_unsat * water_density;
				if (sprop->soilw_zoneCAPIL_unsat - diffCAPIL - soilw_hw0 < 0)
				{
					if (fabs(sprop->soilw_zoneCAPIL_unsat - diffCAPIL - soilw_hw0) > CRIT_PREC)
					{
						printf("ERROR in content_zoneCAPIL calculation in soilEVP.c\n");
						errorCode = 1;
					}
					else
						diffCAPIL = sprop->soilw_zoneCAPIL_unsat - soilw_hw0;
				}
				sprop->soilw_zoneCAPIL_unsat -= diffCAPIL;
			}


		}
	}
	else
		wf->EVPsoilw = 0;



		
	return(errorCode);
}

